﻿--[[
	Simple ligthweight AddOn framework
]]

if KarmaLibrary then return; end

KarmaLibrary = {

}

function KarmaLibrary:GetInstance( inst, name )

	inst = inst or { } 
	
	inst._instance = {
		name = name,
		instance = inst,
		libUtility = KarmaUtility,
		libLibrary = KarmaLibrary,
	}
	
	inst._store = {
		db = nil,
		name = nil,
		defaults = { Options = nil },
	}

	inst._addonmeta = {
		version = GetAddOnMetadata( name, "Version" ), 
		title = GetAddOnMetadata( name, "Title" ),
		author = GetAddOnMetadata( name, "Author" ),
	}
	
	local _inst = inst._instance	
	setmetatable( _inst.libLibrary, { __index = _inst.libUtility } );
	setmetatable( inst, { __index = _inst.libLibrary } );
	
	return inst;
end


function KarmaLibrary:OnEvent( event )
	if event == "VARIABLES_LOADED" then
		self:_DataHookDB( );
		if self.RunVersionFix then self:RunVersionFix()	end;	
		if self.OnVariablesLoaded then self:OnVariablesLoaded() end
	elseif event == "UPDATE_MOUSEOVER_UNIT"	then
		if self.OnUpdateMouse then self:OnUpdateMouse() end
	else	
		if self.OnEventCustom then self:OnEventCustom( event ) end
	end
end


function KarmaLibrary:_DataHookDB( )
	local s = self._store;
	if not s.name then return end
	
	-- init db and chain: self --> db --> defaults
	s.db = _G[s.name] or { }
	_G[s.name] = s.db
	for i,v in pairs(s.defaults) do
		if type(v) == "table"  then
			if not s.db[i] then	s.db[i] = { } end 
			setmetatable( s.db[i], { __index = s.defaults[i] } ); 	
		end
	end
	setmetatable( s.db, { __index = s.defaults } )
end 


function KarmaLibrary:RegisterDB( name, defaults )
	if not name then return end
	local s = self._store;
	
	s.name = name;
	s.defaults = defaults or s.defaults or { };
					
	self:_DataHookDB( );
end 


function KarmaLibrary:ImportToDB( dbitem, src, prefix, isMerge, names )
	if not names or not src then return end
	local dst = self._store.db;
	
	if dbitem then 
		if not dst[dbitem] then dst[dbitem] = { }  end
		dst = dst[dbitem];
	end

	for i,v in pairs(names) do
		local srcname = prefix..v
		if type(i) == "number" then i = v end
		
		if src[srcname] then
			self:SysMsg( "  importing ",srcname," to ",dbitem,":",i );
			if isMerge then
				self:MergeDeep( dst[i], src[srcname] );
			else
				dst[i] = self:CopyDeep( src[srcname] );
			end
		end
	end
end
